@@ -16,6 +16,7 @@ import ctypes
 import errno
 import json
 import os
+import re
 import shutil
 import sys
 import tempfile
@@ -52,7 +53,7 @@ apparmor_fs = "/sys/kernel/security/apparmor"
 
 # IMPORTANT: only raise an AppArmorException if AppArmor is not available,
 #            not on other errors
-def apparmor_available(parser="/sbin/apparmor", apparmor_dirs=None):
+def apparmor_available(parser="/sbin/apparmor_parser", apparmor_dirs=None):
     '''Is AppArmor available for use on this system'''
     dirs = apparmor_dirs
     if dirs is None:
@@ -62,7 +63,7 @@ def apparmor_available(parser="/sbin/apparmor", apparmor_dirs=None):
         if not os.path.isdir(d):
             raise AppArmorException("Could not find '%s'" % d)
 
-    if not os.path.exists(parser):
+    if not os.path.exists(parser):  # pragma: no cover
         rc, parser = easyprof.cmd(['which', 'apparmor_parser'])
         if rc != 0:
             raise AppArmorException("Could not find apparmor_parser")
@@ -70,12 +71,12 @@ def apparmor_available(parser="/sbin/apparmor", apparmor_dirs=None):
 
 def error(out, exit_code=1, do_exit=True):
     '''Print error message and exit'''
-    try:
+    try:  # pragma: no cover
         sys.stderr.write("ERROR: %s\n" % (out))
-    except IOError:
+    except IOError:  # pragma: no cover
         pass
 
-    if do_exit:
+    if do_exit:  # pragma: no cover
         sys.exit(exit_code)
 
 
@@ -85,14 +86,14 @@ def load_profile(profile, parser="/sbin/apparmor_parser",
 
     try:
         apparmor_available(parser)
-    except AppArmorException as e:
+    except AppArmorException as e:  # pragma: nocover
             raise AppArmorException("%s. Skipping load" % e)
 
     command = [parser]
     command.extend(args)
     command.append(profile)
     rc, output = easyprof.cmd(command)
-    if rc != 0:
+    if rc != 0:  # pragma: nocover
         raise AppArmorException("policy load failed with exit status %d: %s" %
                                 (rc, output))
     return (rc, output)
@@ -108,85 +109,88 @@ def load_profiles(profiles, parser="/sbin/apparmor_parser",
 
     try:
         apparmor_available(parser)
-    except AppArmorException as e:
+    except AppArmorException as e:  # pragma: no cover
             raise AppArmorException("%s. Skipping load" % e)
 
     command = [parser]
     command.extend(args)
     command.extend(profiles)
     rc, output = easyprof.cmd(command)
-    if rc != 0:
+    if rc != 0:  # pragma: no cover
         raise AppArmorException("policy load failed with errno %d: %s" %
                                 (rc, output))
     return (rc, output)
 
 
-def unload_profile(profile):
+def _unload_profile(profile):
     '''Unload a profile name (not a filename) from the kernel'''
-
-    try:
-        apparmor_available()
-    except AppArmorException as e:
-            raise AppArmorException("%s. Skipping unload" % e)
-
     apparmor_remove = os.path.join(apparmor_fs, ".remove")
     try:
         with open(apparmor_remove, 'w') as f:
             f.write(profile)
     except (OSError, IOError) as e:
-        if e.errno != errno.ENOENT:
+        if e.errno != errno.ENOENT:  # pragma: no cover
             raise
 
 
-def unload_profiles(profiles):
-    '''Unload a list of profile names (not filenames) from the kernel'''
-
+def unload_profile(profile):
+    '''Unload a profile name from the kernel'''
     try:
         apparmor_available()
-    except AppArmorException as e:
+    except AppArmorException as e:  # pragma: no cover
+        raise AppArmorException("%s. Skipping unload" % e)
+
+    _unload_profile(profile)
+
+
+def unload_profiles(profiles):
+    '''Unload a list of profile names from the kernel'''
+    try:
+        apparmor_available()
+    except AppArmorException as e:  # pragma: no cover
             raise AppArmorException("%s. Skipping unload" % e)
 
-    apparmor_remove = os.path.join(apparmor_fs, ".remove")
     for profile in profiles:
-        try:
-            with open(apparmor_remove, 'w') as f:
-                f.write(profile)
-        except (OSError, IOError) as e:
-            if e.errno != errno.ENOENT:
-                raise
+        _unload_profile(profile)
+
 #
 # End utils
 #
 
 
-def output_policy(easyp, params, count, destdir, force=True):
+def output_policy(easyp, params, count, destdir, force=False, include=None):
     '''Output policy'''
-    # TODO: Adjust easyprof output_policy to behave like this
     policy = easyp.gen_policy(**params)
+
+    # Inject include if it is specified
+    if include is not None:
+        inject_s = '''
+  # injected via click hook
+  #include "%s"
+''' % include
+        policy = re.sub(r'(\s}\s+)$', '\n  %s\\1' % inject_s, policy)
+
     out_fn = None
-    if not destdir:
+    if not destdir:  # pragma: no cover
         if count:
             sys.stdout.write('### aa-easyprof profile #%d ###\n' % count)
         sys.stdout.write('%s\n' % policy)
         return None
     else:
         if 'profile_name' in params:
-            out_fn = AppName(rawname=params['profile_name'])
+            out_fn = AppName(raw_name=params['profile_name'])
         #elif 'binary' in params:
         #    out_fn = params['binary']
-        else:  # should not ever reach this
+        else:  # pragma: no cover
             raise AppArmorException("Could not determine output filename")
 
-        # Generate an absolute path, convertng any path delimiters to '.'
-        out_fn = os.path.join(destdir, out_fn.profilename)
-        # We differ here in that we want to overwrite if exists
-        if not force and os.path.exists(out_fn):
-            raise AppArmorException("'%s' already exists" % out_fn)
+        # Generate an absolute path, converting any path delimiters to '.'
+        out_fn = os.path.join(destdir, out_fn.profile_filename)
 
         if not os.path.exists(destdir):
             os.mkdir(destdir)
 
-        if not os.path.isdir(destdir):
+        if not os.path.isdir(destdir):  # pragma: no cover
             raise AppArmorException("'%s' is not a directory" % destdir)
 
         f, fn = tempfile.mkstemp(prefix='aa-easyprof')
@@ -195,7 +199,19 @@ def output_policy(easyp, params, count, destdir, force=True):
         os.write(f, policy)
         os.close(f)
 
-        shutil.move(fn, out_fn)
+        # Only update if the contents are different
+        policy_orig = "".encode('utf-8')
+        if not force and os.path.exists(out_fn):
+            with open(out_fn) as orig:
+                policy_orig = orig.read()
+                if not isinstance(policy_orig, bytes):
+                    policy_orig = policy_orig.encode('utf-8')
+
+        if force or policy_orig != policy:
+            shutil.move(fn, out_fn)
+            os.chmod(out_fn, 0o644)
+        else:
+            os.unlink(fn)
 
     return out_fn
 
@@ -204,7 +220,7 @@ def walk_up(path):
     while True:
         yield path
         newpath = os.path.dirname(path)
-        if newpath == path:
+        if newpath == path:  # pragma: no cover
             return
         path = newpath
 
@@ -227,20 +243,21 @@ def get_package_manifest(hooklink, pkg):
         if os.path.isdir(info) and os.path.exists(manifest):
             return (manifest, path)
 
-    raise AppArmorException("Could not find click manifest '%s' from " +
-                            "symlink '%s'" % (manifest_name, hooklink))
+    raise AppArmorException("Could not find click manifest " +
+                            "'%s' from symlink '%s'" % (manifest_name,
+                                                        hooklink))
 
 
 def get_click_dir(pkg_dir, pkg, ver):
     '''Find click dir'''
     (p, last) = os.path.split(pkg_dir)
     if not ver == last:
-        raise AppArmorException("package version dir '%s' does not match " +
-                                "expected '%s'" % (pkg_dir, ver))
+        raise AppArmorException("package version dir '%s' " % pkg_dir +
+                                "does not match expected '%s'" % ver)
     (result, last) = os.path.split(p)
     if not pkg == last:
-        raise AppArmorException("package dir '%s' does not match " +
-                                "expected '%s'" % (p, pkg))
+        raise AppArmorException("package dir '%s' does not match " % p +
+                                "expected '%s'" % pkg)
     return result
 
 
@@ -250,8 +267,8 @@ def read_click_manifest(manifest):
     j = json.load(f)
     for field in required_click_fields:
         if not field in j:
-            raise AppArmorException("could not find required field '%s' " +
-                                    "in json" % (field))
+            raise AppArmorException("could not find required field" +
+                                    "'%s' in json" % (field))
     return j
 
 
@@ -305,42 +322,53 @@ class AppName(object):
     _CLICK_PREFIX = "click_"
     _CLICK_SUFFIX = ".json"
 
-    def __init__(self, clickname=None, profilename=None, rawname=None):
+    '''The AppName is used to keep three different views of the name in sync:
+       - click_name: ie, filenames in the hooks directory
+       - profile_filename: ie, filenames in the profiles directory
+       - profile_name (aka raw_name): ie, the AppArmor profile name as loaded
+         into the kernel
+    '''
+
+    def __init__(self, click_name=None, profile_filename=None, raw_name=None):
         # Verify valid click tuple name?
-        if clickname:
-            self.clickname = clickname
-        elif profilename:
-            self.profilename = profilename
-        elif rawname:
-            self._clickname = rawname
+        if click_name:
+            self.click_name = click_name
+        elif profile_filename:
+            self.profile_filename = profile_filename
+        elif raw_name:
+            self._clickname = raw_name
         else:
             raise AppArmorException("AppName object needs an initial value")
 
     @property
-    def clickname(self):
+    def click_name(self):
         return "%s%s" % (self._clickname, self._CLICK_SUFFIX)
 
-    @clickname.setter
-    def clickname(self, clickname):
-        if not clickname.endswith(self._CLICK_SUFFIX):
+    @click_name.setter
+    def click_name(self, click_name):
+        if not click_name.endswith(self._CLICK_SUFFIX):
             raise AppArmorException("invalid click manifest name '%s'" %
-                                    (clickname))
+                                    (click_name))
 
         # strip the suffix off
-        self._clickname = clickname[:-len(self._CLICK_SUFFIX)]
+        self._clickname = click_name[:-len(self._CLICK_SUFFIX)]
 
     @property
-    def profilename(self):
+    def profile_filename(self):
         return "%s%s" % (self._CLICK_PREFIX, self._clickname)
 
-    @profilename.setter
-    def profilename(self, profilename):
-        if not profilename.startswith(self._CLICK_PREFIX):
+    @profile_filename.setter
+    def profile_filename(self, profile_filename):
+        if not profile_filename.startswith(self._CLICK_PREFIX):
             raise AppArmorException("invalid click profile name '%s'" %
-                                    (profilename))
+                                    (profile_filename))
 
         # Strip the click prefix off of the string
-        self._clickname = profilename[len(self._CLICK_PREFIX):]
+        self._clickname = profile_filename[len(self._CLICK_PREFIX):]
+
+    @property
+    def profile_name(self):
+        return "%s" % self._clickname
 
 
 class EasyprofProfile(object):
@@ -434,12 +462,12 @@ def ubuntu_1310_transform(click_manifest):
         if 'template' in profile:
             newp.template = profile['template']
         else:
-            newp.template = "ubuntu-sdk"
+            newp.template = "ubuntu-sdk"  # pragma: no cover
 
         if 'policy_vendor' in profile:
             newp.policyvendor = profile['policy_vendor']
         else:
-            newp.policyvendor = "ubuntu"
+            newp.policyvendor = "ubuntu"  # pragma: no cover
 
         if 'policy_version' in profile:
             # LP: #1214618. Qt json outputs 1.0 as 1. Account for that in our
@@ -447,7 +475,7 @@ def ubuntu_1310_transform(click_manifest):
             # to 1.0.
             newp.policyversion = float(profile['policy_version'])
         else:
-            newp.policyversion = 1.0
+            newp.policyversion = 1.0  # pragma: no cover
 
         newp.add_variable("APP_PKGNAME", click_manifest.app_pkgname)
         newp.add_variable("APP_VERSION", click_manifest.app_version)
@@ -502,7 +530,7 @@ def transform(click_manifest):
 # to_profiles(easyprof_manifest, directory)
 # - directory is the output directory to place it into
 # - returns a list of paths written to
-def to_profiles(easyprof_manifest, directory):
+def to_profiles(easyprof_manifest, directory, include=None, no_verify=False):
     '''Convert manifest to AppArmor profile'''
     (easy_opts, easy_args) = apparmor.easyprof.parse_args(args=[])
 
@@ -515,14 +543,15 @@ def to_profiles(easyprof_manifest, directory):
         count += 1
         try:
             easyp = apparmor.easyprof.AppArmorEasyProfile(binary, options)
-        except AppArmorException as e:
+        except AppArmorException as e:  # pragma: no cover
             error(e.value)
-        except Exception:
+        except Exception:  # pragma: no cover
             raise
 
         params = apparmor.easyprof.gen_policy_params(binary, options)
+        params['no_verify'] = no_verify
         # FIXME: easyp.gen_policy is probably preferred.
-        f = output_policy(easyp, params, count, directory)
+        f = output_policy(easyp, params, count, directory, include=include)
         if directory:
             files.append(f)
 
@@ -535,9 +564,10 @@ def get_missing_profiles(hooksdir, profilesdir):
     '''Find click hooks that are missing profiles'''
     result = []
     for hook in os.listdir(hooksdir):
-        name = AppName(clickname=hook)
-        if not os.path.exists(os.path.join(profilesdir, name.profilename)):
-            result.append(name.clickname)
+        name = AppName(click_name=hook)
+        if not os.path.exists(os.path.join(profilesdir,
+                                           name.profile_filename)):
+            result.append(name.click_name)
     return result
 
 
@@ -548,11 +578,11 @@ def get_missing_clickhooks(hooksdir, profilesdir):
     result = []
     for profile in os.listdir(profilesdir):
         try:
-            name = AppName(profilename=profile)
+            name = AppName(profile_filename=profile)
         except apparmor.click.AppArmorException:
             continue
-        if not os.path.exists(os.path.join(hooksdir, name.clickname)):
-            result.append(name.profilename)
+        if not os.path.exists(os.path.join(hooksdir, name.click_name)):
+            result.append(name.profile_filename)
     return result
 
 
