@@ -21,6 +21,7 @@ import os
 import shutil
 import sys
 import tempfile
+import time
 import unittest
 
 
@@ -92,7 +93,7 @@ class T(unittest.TestCase):
             shutil.rmtree(self.tmpdir)
 
     def test_manifest_name_parsing_1(self):
-        '''simple acceptance test for parsing click manifest name'''
+        '''Test simple acceptance for parsing click manifest name'''
         ex_app = "com.ubuntu.developer.username.myapp"
         ex_name = "myapp"
         ex_vers = "0.1"
@@ -108,7 +109,7 @@ class T(unittest.TestCase):
                                             (ex_vers, version))
 
     def test_bad_manifest_name(self):
-        '''test manifest name with not enough elements'''
+        '''Test manifest name with not enough elements'''
         ex_app = "com.ubuntu.developer.username.myapp"
         ex_name = "myapp"
         ex_vers = "0.1"
@@ -123,7 +124,7 @@ class T(unittest.TestCase):
         raise Exception("name %s should be invalid" % (fname))
 
     def test_bad_manifest_name_2(self):
-        '''test manifest name with too many elements'''
+        '''Test manifest name with too many elements'''
         ex_app = "com.ubuntu.developer.username.myapp"
         ex_name = "myapp"
         ex_vers = "0.1"
@@ -138,28 +139,65 @@ class T(unittest.TestCase):
         raise Exception("name %s should be invalid" % (fname))
 
     def test_click_name_to_profile(self):
-        '''test click name conversion to profile name'''
+        '''Test click name conversion to profile name'''
 
         profile = "com.ubuntu.developer.username.myapp_myapp_0.3"
         expected = "%s.json" % (profile)
         orig = "click_%s" % (profile)
-        app = click.AppName(profilename=orig)
-        self.assertEquals(expected, app.clickname,
+        app = click.AppName(profile_filename=orig)
+        self.assertEquals(expected, app.click_name,
                           "expected click name %s, got %s" % (expected,
-                                                              app.clickname))
+                                                              app.click_name))
+
+    def test_click_appname_no_args_to_init(self):
+        '''Test AppName has args'''
+        try:
+            click.AppName()
+        except click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("name unset should be invalid")
+
+    def test_click_appname_bad_click_name(self):
+        '''Test AppName with bad click_name'''
+        n = "bad"
+        try:
+            click.AppName(click_name="bad")
+        except click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("click_name '%s' should be invalid" % n)
+
+    def test_click_appname_bad_profile_filename(self):
+        '''Test AppName with bad profile_filename'''
+        n = "bad"
+        try:
+            click.AppName(profile_filename="bad")
+        except click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("profile_filename '%s' should be invalid" % n)
 
     def test_profile_name_to_click(self):
-        '''test profile name conversion to click name'''
+        '''Test profile name conversion to click name'''
 
         orig = "com.ubuntu.developer.username.myapp_myapp_0.3.json"
         expected = "click_com.ubuntu.developer.username.myapp_myapp_0.3"
         app = click.AppName(orig)
-        self.assertEquals(expected, app.profilename,
+        self.assertEquals(expected, app.profile_filename,
+                          "expected profile filename %s, got %s" %
+                          (expected, app.profile_filename))
+
+        expected = "com.ubuntu.developer.username.myapp_myapp_0.3"
+        self.assertEquals(expected, app.profile_name,
                           "expected profile name %s, got %s" %
-                          (expected, app.profilename))
+                          (expected, app.profile_name))
 
     def test_find_manifest_file(self):
-        '''test being given a symlink and finding the main package manifest'''
+        '''Test being given a symlink and finding the main package manifest'''
         c = self.clickstate
         c.add_package("package", "version")
         c.add_app("app")
@@ -173,7 +211,7 @@ class T(unittest.TestCase):
                                               (c.pkg_dir, pkg_dir))
 
     def test_parse_security_manifest(self):
-        '''test being given a symlink and parsing the manifests'''
+        '''Test being given a symlink and parsing the manifests'''
         c = self.clickstate
         c.add_package("com.ubuntu.developer.username.myapp", "0.1")
         security_json = '''{
@@ -216,11 +254,30 @@ class T(unittest.TestCase):
                              "Expected %s and %s to differ" %
                              (expected, easyprof_manifest))
 
+    def test_parse_security_manifest_without_required_field(self):
+        '''Test click manifest without required field'''
+        j = dict()
+        j['version'] = "0.1"
+        j['hooks'] = dict()
+        j['title'] = "Some silly demo app"
+        j['maintainer'] = 'Buggy McJerkerson <buggy@bad.soft.warez>'
+
+        self.click_manifest = os.path.join(self.tmpdir, "test-app.manifest")
+        with open(self.click_manifest, "w+") as f:
+            json.dump(j, f, indent=2, separators=(',', ': '), sort_keys=True)
+        try:
+            apparmor.click.read_click_manifest(self.click_manifest)
+        except click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("Should have errored on missing name")
+
     # the top level of the security manifest is not *supposed* to
     # be a single dict entry of the app, but apparently this happens.
     # Accept it and move on
     def test_parse_security_manifest_lenient(self):
-        '''test being given a symlink and parsing the manifests, leniently'''
+        '''Test being given a symlink and parsing the manifests, leniently'''
         c = self.clickstate
         c.add_package("com.ubuntu.developer.username.myapp", "0.3")
         security_json = '''{
@@ -259,7 +316,7 @@ class T(unittest.TestCase):
                           (expected, easyprof_manifest))
 
     def test_parse_security_manifest_with_framework(self):
-        '''test being given a symlink and parsing the manifests w/framework
+        '''Test being given a symlink and parsing the manifests w/framework
            setting'''
         c = self.clickstate
         c.add_package("com.ubuntu.developer.username.myapp", "0.1",
@@ -298,7 +355,7 @@ class T(unittest.TestCase):
                           (expected, easyprof_manifest))
 
     def test_parse_security_manifest_unknown_framework(self):
-        '''test unknown framework raises exception'''
+        '''Test unknown framework raises exception'''
         c = self.clickstate
         c.add_package("com.ubuntu.developer.username.myapp", "0.1",
                       framework='unknown-sdk-13.10')
@@ -322,7 +379,7 @@ class T(unittest.TestCase):
     # if the list inside it is empty. However, I'm seeing click apps
     # without it so we'll accept it and move_on
     def test_parse_security_manifest_no_policy_groups(self):
-        '''test being given a symlink and parsing the manifests, leniently'''
+        '''Test being given a symlink and parsing the manifests, leniently'''
         c = self.clickstate
         package = "com.ubuntu.developer.username.yourapp"
         c.add_package(package, "0.3")
@@ -356,7 +413,7 @@ class T(unittest.TestCase):
                           (expected, easyprof_manifest))
 
     def test_parse_security_manifest_no_entries(self):
-        '''test being given a symlink and parsing the manifests, no policy
+        '''Test being given a symlink and parsing the manifests, no policy
            whatsoever'''
         c = self.clickstate
         package = "com.ubuntu.newbie.username.yourapp"
@@ -391,7 +448,7 @@ class T(unittest.TestCase):
                                                           easyprof_manifest))
 
     def test_parse_security_manifest_unconfined(self):
-        '''test being given a symlink and parsing the manifests for an
+        '''Test being given a symlink and parsing the manifests for an
            unconfined app'''
 
         c = self.clickstate
@@ -755,6 +812,410 @@ class T(unittest.TestCase):
                           "Expected to get %s, got %s" % (expected,
                                                           easyprof_manifest))
 
+    def test_parse_security_manifest_bad_symlink(self):
+        '''Test being given a symlink and parsing the manifests'''
+        c = self.clickstate
+        c.add_package("com.ubuntu.developer.username.myapp", "0.1")
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app('sample-app', manifest=security_json)
+
+        n = "com.ubuntu.developer.username.myapp_sample-app_0.1.json"
+        fn = os.path.join(c.click_dir, n)
+        shutil.move(os.path.dirname(c.click_pkgdir),
+                    os.path.dirname(c.click_pkgdir) + '.new')
+        try:
+            apparmor.click.ClickManifest(fn)
+        except apparmor.click.AppArmorException as e:
+            self.assertTrue("from symlink" in str(e))
+            return
+        except Exception:
+            raise
+        raise Exception("Should have not found click manifest from symlink")
+
+    def test_get_click_dir(self):
+        '''Test get_click_dir()'''
+        c = self.clickstate
+        c.add_package("com.ubuntu.developer.username.myapp", "0.1")
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app('sample-app', manifest=security_json)
+
+        n = "com.ubuntu.developer.username.myapp_sample-app_0.1.json"
+        fn = os.path.join(c.click_dir, n)
+        (package, app, version) = apparmor.click.parse_manifest_name(n)
+        (m, pkg_dir) = apparmor.click.get_package_manifest(fn, package)
+        click_dir = apparmor.click.get_click_dir(pkg_dir, package, version)
+        self.assertTrue(os.path.basename(click_dir) ==
+                        os.path.basename(c.packages_tree),
+                        "'%s' != " % os.path.basename(click_dir) +
+                        "'%s'" % os.path.basename(c.packages_tree))
+
+    def test_get_click_dir_bad_version(self):
+        '''Test get_click_dir() with bad version'''
+        c = self.clickstate
+        c.add_package("com.ubuntu.developer.username.myapp", "0.1")
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app('sample-app', manifest=security_json)
+
+        n = "com.ubuntu.developer.username.myapp_sample-app_0.1.json"
+        fn = os.path.join(c.click_dir, n)
+        (package, app, version) = apparmor.click.parse_manifest_name(n)
+        (m, pkg_dir) = apparmor.click.get_package_manifest(fn, package)
+        click_dir = apparmor.click.get_click_dir(pkg_dir, package, version)
+        self.assertTrue(os.path.basename(click_dir) ==
+                        os.path.basename(c.packages_tree),
+                        "'%s' != " % os.path.basename(click_dir) +
+                        "'%s'" % os.path.basename(c.packages_tree))
+
+        try:
+            apparmor.click.get_click_dir(pkg_dir, package, "0.2")
+        except apparmor.click.AppArmorException as e:
+            self.assertTrue("package version dir" in str(e))
+            return
+        except Exception:
+            raise
+        raise Exception("Should not have found bad version '0.2'")
+
+    def test_get_click_dir_bad_package(self):
+        '''Test get_click_dir() with bad package'''
+        c = self.clickstate
+        c.add_package("com.ubuntu.developer.username.myapp", "0.1")
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app('sample-app', manifest=security_json)
+
+        n = "com.ubuntu.developer.username.myapp_sample-app_0.1.json"
+        fn = os.path.join(c.click_dir, n)
+        (package, app, version) = apparmor.click.parse_manifest_name(n)
+        (m, pkg_dir) = apparmor.click.get_package_manifest(fn, package)
+        click_dir = apparmor.click.get_click_dir(pkg_dir, package, version)
+        self.assertTrue(os.path.basename(click_dir) ==
+                        os.path.basename(c.packages_tree),
+                        "'%s' != " % os.path.basename(click_dir) +
+                        "'%s'" % os.path.basename(c.packages_tree))
+
+        try:
+            apparmor.click.get_click_dir(pkg_dir, package + "-bad", version)
+        except apparmor.click.AppArmorException as e:
+            self.assertTrue("package dir" in str(e))
+            return
+        except Exception:
+            raise
+        raise Exception("Should not have found bad version '%s'" %
+                        package + "-bad")
+
+    def test_to_profiles(self):
+        '''Test to_profiles()'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+        expected_fn = "click_%s" % (full_name)
+        fn = os.path.basename(files[0])
+        self.assertTrue(fn == expected_fn, "'%s' != '%s'" % (fn,
+                                                             expected_fn))
+
+    def test_to_profiles_existing_profile_same(self):
+        '''Test to_profiles() with existing profile that is the same'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+
+        # Test that we don't change the time stamp of the original file if
+        # there are no changes
+        mtime = os.stat(files[0]).st_mtime
+        time.sleep(1)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+
+        cur_mtime = os.stat(files[0]).st_mtime
+        self.assertTrue(mtime == cur_mtime, 'mtime is different')
+
+    def test_to_profiles_existing_profile_different(self):
+        '''Test to_profiles() with existing profile that is different'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+
+        # Test that we do change the time stamp of the original file if
+        # there are changes
+        mtime = os.stat(files[0]).st_mtime
+        time.sleep(1)
+
+        c2 = self.clickstate
+        shutil.rmtree(c2.click_pkgdir)
+        os.unlink(os.path.join(c2.click_dir, "%s.json" % full_name))
+
+        c2.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ ],
+  "policy_version": 1.0
+}'''
+        c2.add_app(appname, manifest=security_json)
+        cm = apparmor.click.ClickManifest(os.path.join(c2.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        expected_fn = "click_%s" % (full_name)
+        fn = os.path.basename(files[0])
+        self.assertTrue(fn == expected_fn, "'%s' != '%s'" % (fn,
+                                                             expected_fn))
+
+        cur_mtime = os.stat(files[0]).st_mtime
+        self.assertTrue(mtime != cur_mtime, 'mtime are the same')
+
+    def test_to_profiles_nonexistent_dir(self):
+        '''Test to_profiles() with non-existent directory'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        didnt_exist_dir = os.path.join(self.tmpdir, "didnt_exist")
+        files = []
+        files = click.to_profiles(easyprof_manifest, didnt_exist_dir)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.isdir(didnt_exist_dir), "Could not find " +
+                        "%s" % didnt_exist_dir)
+
+    def test_to_profiles_with_include(self):
+        '''Test to_profiles() with include file'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        include = os.path.join(self.tmpdir, "test-inject.include")
+        with open(include, "w+") as f:
+            f.write('''  / r,\n''')
+            f.close()
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir, include)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+        expected_fn = "click_%s" % (full_name)
+        fn = os.path.basename(files[0])
+        self.assertTrue(fn == expected_fn, "'%s' != '%s'" % (fn,
+                                                             expected_fn))
+
+        with open(files[0], 'r') as f:
+            contents = f.read()
+            f.close()
+        for s in ['# injected via click hook', 'test-inject.include']:
+            self.assertTrue(s in contents, "Could not find '%s'" % s +
+                            "in:\n%s" % contents)
+
+    def test_to_profiles_without_include(self):
+        '''Test to_profiles() without include file'''
+        c = self.clickstate
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        c.add_package(package, version)
+        security_json = '''{
+  "policy_groups": [ "networking" ],
+  "policy_version": 1.0
+}'''
+        c.add_app(appname, manifest=security_json)
+        full_name = "%s_%s_%s" % (package, appname, version)
+
+        cm = apparmor.click.ClickManifest(os.path.join(c.click_dir,
+                                          "%s.json" % full_name))
+        easyprof_manifest = apparmor.click.transform(cm)
+
+        files = click.to_profiles(easyprof_manifest, self.tmpdir,
+                                  include=None)
+        self.assertTrue(len(files) == 1,
+                        "to_profiles didn't return one profile:\n%s" % files)
+        self.assertTrue(os.path.exists(files[0]), "'%s' does not exist" %
+                                                  files[0])
+        expected_fn = "click_%s" % (full_name)
+        fn = os.path.basename(files[0])
+        self.assertTrue(fn == expected_fn, "'%s' != '%s'" % (fn,
+                                                             expected_fn))
+
+        with open(files[0], 'r') as f:
+            contents = f.read()
+            f.close()
+        for s in ['# injected via click hook', 'test-inject.include']:
+            self.assertFalse(s in contents, "Found '%s' in:\n%s" % (
+                             s, contents))
+
+    def test_easyprof_profile_methods(self):
+        '''Test EasyprofProfile methods'''
+        newp = apparmor.click.EasyprofProfile("foo")
+
+        s = "test-string"
+        self.assertTrue(newp.template is None)
+        newp.template = s
+        self.assertTrue(newp.template == s)
+
+        self.assertTrue(newp.policyvendor is None)
+        newp.policyvendor = s
+        self.assertTrue(newp.policyvendor == s)
+
+        self.assertTrue(newp.policyversion is None)
+        newp.policyversion = 1.0
+        self.assertTrue(newp.policyversion == 1.0)
+
+        k = "test-key"
+        self.assertTrue(k not in newp.profile['template_variables'])
+        newp.add_variable(k, s)
+        self.assertTrue(k in newp.profile['template_variables'])
+        self.assertTrue(newp.profile['template_variables'][k] == s)
+
+        e = "test-entry"
+        self.assertTrue(e not in newp.profile['policy_groups'])
+        newp.add_policygroup(e)
+        self.assertTrue(e in newp.profile['policy_groups'])
+
+        self.assertTrue('abstractions' not in newp.profile)
+        newp.add_abstraction(e)
+        self.assertTrue(e in newp.profile['abstractions'])
+
+        self.assertTrue('read_path' not in newp.profile)
+        newp.add_read_path(e)
+        self.assertTrue(e in newp.profile['read_path'])
+
+        self.assertTrue('write_path' not in newp.profile)
+        newp.add_write_path(e)
+        self.assertTrue(e in newp.profile['write_path'])
+
+    def test_parse_manifest_name(self):
+        '''Test parse_manifest_name()'''
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        manifest_name = "%s_%s_%s.json" % (package, appname, version)
+
+        (p, a, v) = apparmor.click.parse_manifest_name(manifest_name)
+        self.assertTrue(p == package, "'%s' != '%s'" % (p, package))
+        self.assertTrue(a == appname, "'%s' != '%s'" % (a, appname))
+        self.assertTrue(v == version, "'%s' != '%s'" % (v, version))
+
+    def test_parse_manifest_name_bad_no_json(self):
+        '''Test parse_manifest_name() without .json'''
+        package = "com.ubuntu.developer.username.yourapp"
+        appname = "sample-app"
+        version = "0.3"
+        manifest_name = "%s_%s_%s" % (package, appname, version)
+
+        try:
+            apparmor.click.parse_manifest_name(manifest_name)
+        except apparmor.click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("'%s' should be invalid" % (manifest_name))
+
+    def test_parse_manifest_name_bad_name(self):
+        '''Test parse_manifest_name() bad name'''
+        package = "com.ubuntu.developer.username.yourapp"
+        manifest_name = "%s.json" % package
+
+        try:
+            apparmor.click.parse_manifest_name(manifest_name)
+        except apparmor.click.AppArmorException:
+            return
+        except Exception:
+            raise
+        raise Exception("'%s' should be invalid" % (manifest_name))
+
 
 class AppArmorManifestSynchronization(unittest.TestCase):
     def setUp(self):
@@ -767,7 +1228,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
             shutil.rmtree(self.tmpdir)
 
     def test_two_equal_directories(self):
-        '''test two equal directories'''
+        '''Test two equal directories'''
         c = self.clickstate
         clicks = ["alpha_beta_gamma", "click_click_version",
                   "wat_no-really_wat"]
@@ -788,7 +1249,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                           "Expected to get no click hooks, got %s" % (result))
 
     def test_two_empty_directories_for_profiles(self):
-        '''test two empty directories returns no new needed profiles'''
+        '''Test two empty directories returns no new needed profiles'''
         c = self.clickstate
 
         expected = []
@@ -797,7 +1258,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                           "Expected to get no profiles, got %s" % (result))
 
     def test_two_empty_directories_for_clicks(self):
-        '''test two empty directories returns no removed click hooks'''
+        '''Test two empty directories returns no removed click hooks'''
         c = self.clickstate
 
         expected = []
@@ -806,7 +1267,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                           "Expected to get no clickhooks, got %s" % (result))
 
     def test_versus_empty_profiles_directory(self):
-        '''test against empty profiles directory'''
+        '''Test against empty profiles directory'''
         c = self.clickstate
         clicks = ["alpha_beta_gamma.json", "click_click_version.json",
                   "wat_no-really_wat.json"]
@@ -821,7 +1282,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                                                                    result))
 
     def test_versus_empty_clicks_directory(self):
-        '''test against empty clicks directory'''
+        '''Test against empty clicks directory'''
         c = self.clickstate
         clicks = ["alpha_beta_gamma", "click_click_version",
                   "wat_no-really_wat"]
@@ -837,7 +1298,7 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                                                                 result))
 
     def test_two_unequal_directories(self):
-        '''test two equal directories'''
+        '''Test two equal directories'''
         c = self.clickstate
         expected_clicks = ['missing_click_profile.json',
                            'another-missing_click_profile.json']
@@ -865,6 +1326,24 @@ class AppArmorManifestSynchronization(unittest.TestCase):
                           "Expected to get '%s' missing click hooks, got %s" %
                           (set(expected_profiles), result))
 
+    def test_two_unequal_directories_bad_profile_filename(self):
+        '''Test two unequal directories (bad profile_filename)'''
+        c = self.clickstate
+        clicks = ["alpha_beta_gamma", "click_click_version",
+                  "wat_no-really_wat"]
+        for cname in clicks:
+            with open(os.path.join(c.click_dir, '%s.json' %
+                      (cname)), 'w+') as f:
+                f.write('invalid json here')
+            with open(os.path.join(c.profiles_dir, 'badformat_%s' %
+                                   (cname)), 'w+') as f:
+                f.write('profile %s { }' % (cname))
+
+        expected = []
+        result = click.get_missing_clickhooks(c.click_dir, c.profiles_dir)
+        self.assertEquals(expected, result,
+                          "Expected to get no click hooks, got %s" % (result))
+
 
 class AppArmorPolicyModificationTests(unittest.TestCase):
 
@@ -878,7 +1357,7 @@ class AppArmorPolicyModificationTests(unittest.TestCase):
                 f.write(p)
 
     def test_load_simple_policy(self):
-        '''test basic load policy function'''
+        '''Test basic load policy function'''
         policy = "profile invalid_policy_dont_use { }"
         with tempfile.NamedTemporaryFile(prefix="aa-clicktest-",
                                          delete=False) as profile:
@@ -894,7 +1373,7 @@ class AppArmorPolicyModificationTests(unittest.TestCase):
         os.remove(name)
 
     def test_load_simple_policies(self):
-        '''test basic load polices function'''
+        '''Test basic load policies function'''
         policy = "profile invalid_policy_dont_use { }"
         with tempfile.NamedTemporaryFile(prefix="aa-clicktest-",
                                          delete=False) as profile:
@@ -909,7 +1388,15 @@ class AppArmorPolicyModificationTests(unittest.TestCase):
         self.remove_profiles.append('invalid_policy_dont_use')
         os.remove(name)
 
+    def test_load_no_policies(self):
+        '''Test load zero policies function'''
+        try:
+            rc, output = click.load_profiles([])
+        except:
+            raise
+
     def test_unload_simple_policy(self):
+        '''Test unload simple policy'''
         policy = "profile invalid_policy_dont_use { }"
         with tempfile.NamedTemporaryFile(prefix="aa-clicktest-",
                                          delete=False) as profile:
@@ -925,6 +1412,7 @@ class AppArmorPolicyModificationTests(unittest.TestCase):
         os.remove(name)
 
     def test_unload_simple_policies(self):
+        '''Test unload simple policies'''
         policies = ['more_invalid_policy_%s' % (suffix)
                     for suffix in ['blart', 'blort', 'blat']]
         names = []
@@ -945,15 +1433,25 @@ class AppArmorPolicyModificationTests(unittest.TestCase):
             os.remove(name)
 
     def test_unload_nonexistent_policy(self):
+        '''Test unload nonexistent policy'''
         profile = 'invalid_policy_does_not_exist_dont_use'
         click.unload_profile(profile)
 
+    def test_apparmor_available(self):
+        '''Test apparmor_available()'''
+        try:
+            click.apparmor_available(parser="./aa-clickhook",
+                                     apparmor_dirs=['./'])
+        except click.AppArmorException:
+            raise Exception("apparmor_available() should have passed")
+
     def test_apparmor_not_available(self):
         '''Test apparmor not available'''
         try:
             click.apparmor_available(parser="/nonexistent",
                                      apparmor_dirs=['/nonexistent.dir'])
-        except click.AppArmorException:
+        except click.AppArmorException as e:
+            self.assertTrue("Could not find '/nonexistent.dir'" in str(e))
             return
         except Exception:
             raise
